using System;

namespace Indexer.Engine.Parsers
{
	public class MailParser: ParserBase
	{
		public MailParser(): base(".eml")
		{
		}
		
		protected override bool GetSupportsCopy()
		{
			return true;
		}
		
		private string[] _signatures = new string[]{"X-Opera-Status: ", "From ", "From: ", "Date: ", "Subject: ", "To: ", "Return-Path: ", "Received: ", "Message-ID: "};
		
		public override bool Accepts(string fileName, byte[] previewBuffer)
		{
			return Accepts(fileName, BufferToString(previewBuffer, false));
		}
		
		public override bool Accepts(string fileName, string previewBuffer)
		{
			previewBuffer = "\n" + previewBuffer;
			int hits = 0;
			foreach(string signature in _signatures)
			{
				if (previewBuffer.Contains("\n" + signature))
				{
					hits++;
				}
			}
			return hits >= 3;	// TODO: TUNE:
		}
		
		private static bool IsBlank(string line)
		{
			if (String.IsNullOrEmpty(line))
			{
				return true;
			}
			else
			{
				return String.IsNullOrEmpty(line.Trim());
			}
		}
		
		public override void Copy(string fileName, System.IO.TextWriter output)
		{
			System.IO.TextReader reader = System.Text.FileStream.CreateReader(fileName);
			try
			{
				Writer writer = new Writer(output);
				try
				{
					string line;
					while ((line = reader.ReadLine()) != null)
					{
						if (String.Compare(line, "Content-Transfer-Encoding: base64", true) == 0)
						{
							writer.WriteLine(line);
							while ((line = reader.ReadLine()) != null)
							{
								if (!IsBlank(line))
								{
									writer.WriteLine(line);
								}
								else
								{
									break;
								}
							}
							writer.WriteLine();
							writer.WriteLine("...base64 content comes here");
							
							bool skipEmptyLines = true;
							while ((line = reader.ReadLine()) != null)
							{
								if (IsBlank(line))
								{
									if (skipEmptyLines)
									{
									}
									else
									{
										writer.WriteLine(line);
										break;
									}
								}
								else if (line.StartsWith("-----------"))	// TODO: is it correct detection of the content separator?
								{
									writer.WriteLine();
									writer.WriteLine(line);
									break;
								}
								else
								{
									skipEmptyLines = false;
								}
							}
						}
						else
						{
							writer.WriteLine(line);
						}
					}
				}
				finally
				{
					writer.Close();
				}
			}
			finally
			{
				reader.Close();
			}
		}
	}
}
