using System;

namespace Explorer.Engine
{
	public abstract class FileSystemBase: IFileSystem
	{
		private Counter releaseCounter = new Counter();
		
		public void AddRef()
		{
			releaseCounter.AddRef();
		}
		
		public bool Release()
		{
			if (releaseCounter.Release())
			{
				Dispose();
				return true;
			}
			else
			{
				return false;
			}
		}
		
		public FileSystemBase()
		{
		}
		
		public virtual Explorer.Model.Entries CreateList(string folder)
		{
			return new Model.Entries();
		}
		
		public virtual Explorer.Model.Entry GetInfo(string entryName)
		{
			return null;
		}
		
		public virtual void CreateDirectory(string folderName)
		{
			throw new NotImplementedException();
		}
		
		public virtual void DeleteFile(string fileName)
		{
			throw new NotImplementedException();
		}
		
		public virtual void DeleteDirectory(string folderName)
		{
			throw new NotImplementedException();
		}
		
		public virtual System.IO.Stream OpenFile(string fileName, System.IO.FileMode mode, System.IO.FileAccess access, System.IO.FileShare share)
		{
			throw new NotImplementedException();
		}

		public virtual string GetBaseFileName(string fileName, out IRawFileSystem container)
		{
			container = this;
			return fileName;
		}

		public virtual void GetContainerInfo(string path, out string container, out string containerPath)
		{
			container = null;
			containerPath = null;
		}
		
		public abstract void Open();
		
		protected virtual void Close()
		{
		}
		
		private bool _disposed = false;
		public void Dispose()
		{
			if (!_disposed)
			{
				_disposed = true;
				Close();
			}
		}
	}
}
