using System;

namespace Explorer.Engine
{
	public class MirrorFileSystem: FileSystemBase
	{
		private IFileSystem _base;
		private NameMapper _mapper;
		
		public MirrorFileSystem(char drive, IFileSystem fileSystem, string root)
		{
			_mapper = new NameMapper(drive.ToString().ToUpper() + ":\\", root);
			_base = fileSystem;
			_base.AddRef();
		}
		
		public override void Open()
		{
			if (_base != null)
			{
				_base.Open();
			}
		}
		
		protected override void Close()
		{
			base.Close();
			if (_base != null)
			{
				_base.Release();
				_base = null;
			}
		}

		private Model.Entries CreateList()
		{
			Model.Entries result = new Model.Entries();
			Model.Entry item = new Model.Entry();
			item.Kind = Model.EntryKind.Folder;
			item.Name = _mapper.OuterPrefix;
			result.Add(item);
			return result;
		}
		
		public override Explorer.Model.Entries CreateList(string folder)
		{
			if (String.IsNullOrEmpty(folder))
			{
				return CreateList();
			}
			
			folder = _mapper.OuterToInner(folder);
			Model.Entries result = _base.CreateList(folder);
			foreach(Model.Entry entry in result)
			{
				_mapper.InnerToOuter(entry);
			}
			return result;
		}

		public override System.IO.Stream OpenFile(string fileName, System.IO.FileMode mode, System.IO.FileAccess access, System.IO.FileShare share)
		{
			return _base.OpenFile(_mapper.OuterToInner(fileName), mode, access, share);
		}
		
		public override Explorer.Model.Entry GetInfo(string entryName)
		{
			Model.Entry result = _base.GetInfo(_mapper.OuterToInner(entryName));
			if (result != null)
			{
				_mapper.InnerToOuter(result);
			}
			return result;
		}
		
		public override void CreateDirectory(string folderName)
		{
			_base.CreateDirectory(_mapper.OuterToInner(folderName));
		}
		
		public override void DeleteDirectory(string folderName)
		{
			_base.DeleteDirectory(_mapper.OuterToInner(folderName));
		}
		
		public override void DeleteFile(string fileName)
		{
			_base.DeleteFile(_mapper.OuterToInner(fileName));
		}
	}
}
