using System;

namespace Explorer.Engine
{
	public class NameMapper
	{
		public NameMapper(string outerPrefix, string innerPrefix)
		{
			this.OuterPrefix = outerPrefix;
			this.InnerPrefix = innerPrefix;
		}
		
		private string _outerPrefix;
		public string OuterPrefix
		{
			get
			{
				return _outerPrefix;
			}
			set
			{
				_outerPrefix = Model.Tools.FileNameToFolderName(value);
			}
		}
		
		private string _innerPrefix;
		public string InnerPrefix
		{
			get
			{
				return _innerPrefix;
			}
			set
			{
				_innerPrefix = Model.Tools.FileNameToFolderName(value);
			}
		}

		#if DEBUG
		private void CheckPrefix(string path, string prefix)
		{
			bool ok = true;
			if (path.Length < prefix.Length)
			{
				ok = false;
			}
			else if (String.Compare(path.Substring(0, prefix.Length), prefix, true) != 0)
			{
				ok = false;
			}
			if (!ok)
			{
				throw new ArgumentOutOfRangeException(String.Format("path \"{0}\" does not start with \"{1}\" prefix", path, prefix));
			}
		}
		#endif
		
		public string OuterToInner(string path)
		{
			#if DEBUG
			CheckPrefix(path, _outerPrefix);
			#endif
			return _innerPrefix + path.Substring(_outerPrefix.Length);
		}
		
		public string InnerToOuter(string path)
		{
			#if DEBUG
			CheckPrefix(path, _innerPrefix);
			#endif
			return _outerPrefix + path.Substring(_innerPrefix.Length);
		}
		
		public void InnerToOuter(Model.Entry entry)
		{
			if (entry.Path.Length < _innerPrefix.Length)
			{
				// this is probably root folder jumping out of the mapped space
				#if DEBUG
				if (entry.Kind == Model.EntryKind.File)
				{
					throw new ArgumentException();
				}
				CheckPrefix(Model.Tools.FileNameToFolderName(entry.FullName), _innerPrefix);
				#endif
				entry.Path = _outerPrefix;
				entry.Name = "";
			}
			else
			{
				entry.Path = InnerToOuter(entry.Path);
			}
		}
	}
}
