using System;

namespace Explorer.Engine
{
	public class NativeFileSystem: FileSystemBase
	{
		public override void Open()
		{
		}
		
		public override Explorer.Model.Entries CreateList(string folder)
		{
			return NativeUtils.CreateList(folder);
		}
		
		public override Model.Entry GetInfo(string entryName)
		{
			try
			{
				return NativeUtils.GetInfo(entryName);
			}
			catch(Exception)
			{
				return null;
			}
		}
		
		public override void CreateDirectory(string folderName)
		{
			System.IO.Directory.CreateDirectory(folderName);
		}
		
		public override void DeleteFile(string fileName)
		{
			System.IO.File.Delete(fileName);
		}
		
		public override void DeleteDirectory(string folderName)
		{
			System.IO.Directory.Delete(folderName);
		}
		
		public override System.IO.Stream OpenFile(string fileName, System.IO.FileMode mode, System.IO.FileAccess access, System.IO.FileShare share)
		{
			return System.IO.File.Open(fileName, mode, access, share);
		}
		
		public override string GetBaseFileName(string fileName, out IRawFileSystem container)
		{
			container = null;
			return fileName;
		}
	}
	
	public class NativeUtils
	{
		public static Model.Entries CreateList()
		{
			Model.Entries result = new Model.Entries();
			foreach(System.IO.DriveInfo drive in System.IO.DriveInfo.GetDrives())
			{
				Model.Entry item = new Model.Entry();
				try
				{
					item.Kind = Model.EntryKind.Folder;
					item.Name = drive.Name;
					result.Add(item);
				}
				catch(Exception)
				{
				}
			}
			return result;
		}
		
		public static Model.Entry GetInfo(string entryName)
		{
			if (String.IsNullOrEmpty(entryName))
			{
				return null;
			}
			
			if (System.IO.File.Exists(entryName))
			{
				return GetInfo(new System.IO.FileInfo(entryName));
			}
			else if (System.IO.Directory.Exists(entryName))
			{
				return GetInfo(new System.IO.DirectoryInfo(entryName));
			}
			else
			{
				return null;
			}
		}

		private static string GetPath(string fullName)
		{
			if (String.IsNullOrEmpty(fullName))
			{
				return "";
			}
			
			if (fullName.EndsWith(":\\"))
			{
				return "";
			}
			
			if (fullName.EndsWith("\\"))
			{
				fullName = fullName.Substring(0, fullName.Length - 1);
			}
			
			int i = fullName.LastIndexOf("\\");
			if (i < 0)
			{
				return "";
			}
			else
			{
				return fullName.Substring(0, i + 1);
			}
		}
		
		public static Model.Entry GetInfo(System.IO.FileInfo info)
		{
			Model.Entry item = new Model.Entry();
			item.Kind = Model.EntryKind.File;
			item.Name = info.Name;
			item.Size = info.Length;
			item.LastWriteTime = info.LastWriteTime;
			item.Path = GetPath(info.FullName);
			item.Attributes = info.Attributes;
			return item;
		}

		public static Model.Entry GetInfo(System.IO.DirectoryInfo info)
		{
			Model.Entry item = new Model.Entry();
			item.Kind = Model.EntryKind.Folder;
			item.Name = info.Name;
			item.LastWriteTime = info.LastWriteTime;
			item.Path = GetPath(info.FullName);
			item.Attributes = info.Attributes;
			return item;
		}
		
		public static Model.Entries CreateList(string folder)
		{
			if (String.IsNullOrEmpty(folder))
			{
				return CreateList();
			}
			
			if (!folder.EndsWith("\\"))
			{
				folder = folder + "\\";
			}
			
			Model.Entries result = new Model.Entries();

			{
				Model.Entry item = new Model.Entry();
				item.Kind = Model.EntryKind.UpFolder;
				item.Path = folder;
				result.Add(item);
			}
			
			try
			{
				System.IO.DirectoryInfo root = new System.IO.DirectoryInfo(folder);
				foreach(System.IO.DirectoryInfo info in root.GetDirectories())
				{
					result.Add(GetInfo(info));
				}
				foreach(System.IO.FileInfo info in root.GetFiles())
				{
					result.Add(GetInfo(info));
				}
			}
			catch(Exception)
			{
			}
			return result;			
		}
	}
}
