using System;

namespace Microsoft.Win32
{
	/* examples using VBReg

	 // write to default value
	VBReg.Write("HKCU\\Software\\First Information Systems\\',3)

	// create keys and write text value
	VBReg.Write("HKLM\\Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\ActiveChart\\DisplayName","hallo 3D")

	// remove associations for files with KUK extension. (both ext key and its shell
	// definition key)
	if (VBReg.Exists("HKCR\\.kuk\\")
	{
		VBReg.Delete("HKCR\\"+VBReg.Read("HKCR\\.kuk\\")+"\\");
		VBReg.Delete("HKCR\\.kuk\\");
	}

	// delete value
	VBReg.Delete("HKCU\\Software\\First Information Systems\\ActiveChart\\OpenFilePath")
	// delete value by assignment
	VBReg.Write("HKCU\\Software\\First Information Systems\\ActiveChart\\OpenFilePath",null)
	*/
	public class VBReg
	{
		public static object Read(string path)
		{
			return Read(path, null);
		}

		public static object Read(string path, object defaultValue)
		{
			try
			{
				object result = new TReg(path).GetValue();
				return (result == null) ? defaultValue: result;
			}
			catch(Exception)
			{
				return defaultValue;
			}
		}

		public static string ReadString(string path, string defaultValue)
		{
			object result = Read(path, defaultValue);
			return (result == null) ? null : System.Convert.ToString(result);
		}

		public static string ReadString(string path)
		{
			return ReadString(path, null);
		}

		public static bool ReadBoolean(string path, bool defaultValue)
		{
			return System.Convert.ToBoolean(Read(path, defaultValue));
		}

		public static bool ReadBoolean(string path)
		{
			return ReadBoolean(path, false);
		}

		public static Int32 ReadInt32(string path, Int32 defaultValue)
		{
			return System.Convert.ToInt32(Read(path, defaultValue));
		}

		public static Int32 ReadInt32(string path)
		{
			return ReadInt32(path, 0);
		}

		public static bool Write(string path, object value)
		{
			try
			{
				new TReg(path).SetValue(value);
				return true;
			}
			catch(Exception)
			{
				return false;
			}
		}

		public static bool Delete(string path)
		{
			try
			{
				new TReg(path).Delete();
				return true;
			}
			catch(Exception)
			{
				return false;
			}
		}

		public static bool Exists(string path)
		{
			try
			{
				return new TReg(path).Exists();
			}
			catch(Exception)
			{
				return false;
			}
		}
	}

	// this is a VB helper class. after instantiation just one of declared functions
	// may be run.
	internal class TReg
	{
		private string path;			// key path witin the root key
		private string name;			// value name or ""
		private RegistryKey key;		// root registry key or null
		private string keyName;			// keyName + path gives full path

		private static void Parse(string APath, ref string path, ref RegistryKey key, ref string keyName, ref string name)
		{
			int p;
			path = APath;
			p = path.IndexOf("\\");
			key = null;
			keyName = "";
			if (p > 0)
			{
				name = path.Substring(0, p).ToUpper();
				path = path.Substring(p + 1);

				keyName = name + "\\";
				switch(name)
				{
					case "HKCR":
						key = Registry.ClassesRoot;
						break;

					case "HKCU":
						key = Registry.CurrentUser;
						break;

					case "HKLM":
						key = Registry.LocalMachine;
						break;

					case "HKU":
						key = Registry.Users;
						break;
				}
			}

			name = "";
			p = path.IndexOf("\\\\");
			if (p >= 0)
			{
				name = path.Substring(p + 2);
				path = path.Substring(0, p);
			}
			else
			{
				for(p = path.Length - 1; p >= 0; p--)
				{
					if (path[p] == '\\')
					{
						name = path.Substring(p + 1);
						path = path.Substring(0, p);
						break;
					}
				}
			}
		}

		public TReg(string APath)
		{
			Parse(APath, ref path, ref key, ref keyName, ref name);
		}

		public bool Exists()
		{
			if (key == null) return false;
			if (name != "") return GetValue() != null;
			RegistryKey r = key.OpenSubKey(path, false);
			if (r == null) return false;
			r.Close();
			return true;
		}

		public bool Delete()
		{
			if (key == null) return false;
			if (name != "") return SetValue(null);
			key.DeleteSubKeyTree(path);
			return true;
		}

		public object GetValue()
		{
			if (key == null) return null;
			RegistryKey r = key.OpenSubKey(path, false);
			if (r == null) return null;
			try
			{
				return r.GetValue(name);
			}
			finally
			{
				r.Close();
			}
		}

		public bool SetValue(object value)
		{
			if (key == null) return false;
			RegistryKey r = key.CreateSubKey(path);
			if (r == null) return false;
			try
			{
				if (value == null)
				{
					if (name == "")
					{
						r.SetValue(name, "");
					}
					else
					{
						r.DeleteValue(name);
					}
				}
				else
				{
					r.SetValue(name, value);
				}
				return true;
			}
			finally
			{
				r.Close();
			}
		}
	}

}
