using System;

namespace Prolog.Memory
{
	public interface ICell: ICloneable
	{
		string Name { get; set; }
		string FunctorId { get; set; }
		int Arity { get; set; }
		ICell this[int index] { get; set; }
	}
	
	public class Functor: ICell
	{
		private string _name;
		private ICell[] _arguments;
		
		public Functor(string name)
		{
			this.FunctorId = name;
		}
		
		public Functor(string name, int arity)
		{
			_name = name;
			if (arity > 0)
			{
				this.Arity = arity;
			}
		}
		
		public string Name
		{
			get
			{
				return _name;
			}
			set
			{
				_name = value;
			}
		}
		
		public string FunctorId
		{
			get
			{
				return String.Format("{0}/{1}", Name, Arity);
			}
			set
			{
				int i = value.LastIndexOf('/');
				if (i < 0)
				{
					Name = value;
					Arity = 0;
				}
				else
				{
					Name = value.Substring(0, i);
					Arity = int.Parse(value.Substring(i + 1));
				}
			}
		}
		
		public int Arity
		{
			get
			{
				if (_arguments == null)
				{
					return 0;
				}
				else
				{
					return _arguments.Length;
				}
			}
			set
			{
				int oldValue = Arity;
				if (value != oldValue)
				{
					if (value == 0)
					{
						_arguments = null;
					}
					else if (_arguments == null)
					{
						_arguments = new ICell[value];
					}
					else
					{
						System.Array.Resize(ref _arguments, value);
					}
				}
			}
		}
		
		public ICell this[int index]
		{
			get
			{
				return _arguments[index];
			}
			set
			{
				_arguments[index] = value;
			}
		}
		
		public object Clone()
		{
			ICell result = new Functor(Name, Arity);
			for(int i = 0; i < Arity; i++)
			{
				ICell item = this[i];
				if (item != null)
				{
					result[i] = (ICell)item.Clone();
				}
			}
			return result;
		}
	}
}
