using System;
using System.Collections;
using System.Reflection;

namespace Quest.Atomic
{
	/// <summary>
	/// Description of AtomInfo.
	/// </summary>
	public class AtomInfo
	{
		public enum FieldFlags
		{
			Internal = 01,
			Editable = 02,
			Linkable = 04,

			All = Internal | Editable | Linkable
		};

		public static FieldInfo[] GetFields(Type factory, FieldFlags flags, string[] names)
		{
			FieldInfo[] allFields = factory.GetFields();
			ArrayList fields = new ArrayList();

			for(int i = 0; i < allFields.Length; i++)
			{
				FieldInfo info = allFields[i];
				if((!info.IsPublic) || info.IsLiteral || info.IsInitOnly || info.IsNotSerialized) continue;

				if(names != null)
				{
					bool found = false;
					foreach(string name in names)
					{
						if(String.Compare(info.Name, name, true) == 0)
						{
							found = true;
							break;
						}
					}
					if(!found) continue;
				}

				// process internal fields
				switch(info.Name)
				{
					case "ID":
						// this is internal field
						if((flags & FieldFlags.Internal) == FieldFlags.Internal)
						{
							fields.Add(info);
						}
						continue;
				}

				// process linkable fields
				if(info.FieldType.IsSubclassOf(typeof(Atom)))
				{
					// this is linkable field
					if((flags & FieldFlags.Linkable) == FieldFlags.Linkable)
					{
						fields.Add(info);
					}
					continue;
				}

				// it is editable field
				if((flags & FieldFlags.Editable) == FieldFlags.Editable)
				{
					fields.Add(info);
				}
				continue;
			}

			FieldInfo[] result = new FieldInfo[fields.Count];
			for(int i = 0; i < fields.Count; i++)
			{
				result[i] = (FieldInfo)fields[i];
			}
			return result;
		}

		public static FieldInfo[] GetFields(Type factory, FieldFlags flags)
		{
			return GetFields(factory, flags, null);
		}

		public static FieldInfo[] GetFields(Atom atom, FieldFlags flags)
		{
			// TODO: speed optimize by caching result in atom.Owner
			return GetFields(atom.GetType(), flags, null);
		}
	}
}
