using System;
using System.Collections;

namespace Quest.Atomic
{
	/// <summary>
	/// Description of Binder.
	/// </summary>
	[Serializable]
	public class Binder: ArrayList
	{
		#region Delayed update
		internal class UpdateRequest
		{
			public Atom Atom;
			public System.Reflection.FieldInfo FieldInfo;
			public int ID;
		}

		private int updateLevel = 0;
		private ArrayList updateRequests = null;

		public void BeginUpdate()
		{
			if(updateLevel == 0)
			{
				updateRequests = new ArrayList();
			}
			updateLevel++;
		}

		public void EndUpdate()
		{
			updateLevel--;
			if(updateLevel == 0)
			{
				try
				{
					foreach(UpdateRequest request in updateRequests)
					{
						Atom result = Bind(request.FieldInfo.FieldType, request.ID);
						if(result != null)
						{
							request.FieldInfo.SetValue(request.Atom, result);
						}
					}
				}
				finally
				{
					updateRequests.Clear();
					updateRequests = null;
				}
			}
		}

		private void AddUpdateRequest(Atom atom, System.Reflection.FieldInfo field, int anID)
		{
			UpdateRequest request = new UpdateRequest();
			request.Atom = atom;
			request.FieldInfo = field;
			request.ID = anID;
			updateRequests.Add(request);
		}
		#endregion

		public new AtomCollection this[int index]
		{
			get
			{
				return (AtomCollection)base[index];
			}
		}

		public AtomCollection Bind(Type aFactory)
		{
			for(int i = 0; i < Count; i++)
			{
				AtomCollection result = this[i];
				if(result.Factory == aFactory)
					return result;
			}
			return null;
		}

		public AtomCollection Bind(string name)
		{
			for(int i = 0; i < Count; i++)
			{
				AtomCollection result = this[i];
				if(result.Name == name)
					return result;
			}
			return null;
		}

		public Atom Bind(Type aFactory, int anID)
		{
			AtomCollection factory = Bind(aFactory);
			return (factory == null) ? null : factory[anID];
		}

		public Atom Bind(Atom atom, System.Reflection.FieldInfo field, int anID)
		{
			Atom result = null;
			if(anID != AtomCollection.NullID)
			{
				if(updateLevel > 0)
				{
					result = Bind(field.FieldType, anID);
					if(result == null)
					{
						AddUpdateRequest(atom, field, anID);
					}
					else
					{
						field.SetValue(atom, result);
					}
				}
				else
				{
					result = Bind(field.FieldType, anID);
					if(result != null)
					{
						field.SetValue(atom, result);
					}
				}
			}
			return result;
		}
	}
}
