using System;
using System.Data;

namespace Quest.Atomic
{
	/// <summary>
	/// Description of Database.
	/// </summary>
	public class Database: IDisposable
	{
		protected Binder Binder = new Binder();
		protected string pHome = null;
		private int pCodePage = Csv.DefaultCodePage;

		public bool ReadOnly = false;

		public Binder Collections
		{
			get
			{
				return Binder;
			}
		}

		public Atom New(Type aFactory)
		{
			AtomCollection factory = Binder.Bind(aFactory);
			return (factory == null) ? null : factory.New();
		}

		public Database()
		{
		}

		public int CodePage
		{
			get
			{
				return pCodePage;
			}
			set
			{
				pCodePage = value;
				foreach(AtomCollection factory in Binder)
				{
					factory.CodePage = pCodePage;
				}
			}
		}

		#region DataFile routines
		private string pDataFileName = null;
		private System.IO.Compression.ICompressedFile pStorage = null;
		public string DataFileName
		{
			get
			{
				return pDataFileName;
			}
			set
			{
				if (pStorage != null)
				{
					if (!pStorage.ReadOnly)
					{
						pStorage.ReadOnly = ReadOnly;
					}
					pStorage.Dispose();
					pStorage = null;
				}
				pDataFileName = String.IsNullOrEmpty(value) ? null : value;
				pStorage = OpenStorage(pDataFileName, System.IO.FileMode.OpenOrCreate);
				Home = pStorage.Folder + "\\";  // TODO: use some System.IO.Path.Combine
			}
		}

		private System.IO.Compression.ICompressedFile OpenStorage(string fileName, System.IO.FileMode mode)
		{
			if (String.IsNullOrEmpty(fileName))
			{
				return new System.IO.Compression.NullFile();
			}
			else
			{
				return new System.IO.Compression.ZipFile(fileName, mode, System.IO.FileAccess.ReadWrite /* TODO: get it as argument */);
			}
		}

		public void SetOutputDataFileName(string newDataFileName)
		{
			System.IO.Compression.ICompressedFile newStorage = OpenStorage(newDataFileName, System.IO.FileMode.Create);
			string[] files = System.IO.Directory.GetFiles(Home);

			foreach (string file in files)
			{
				System.IO.File.Copy(file, System.IO.Path.Combine(newStorage.Folder, System.IO.Path.GetFileName(file)));
			}

			if (pStorage != null)
			{
				pStorage.ReadOnly = true;
				pStorage.Dispose();
				pStorage = null;
			}

			pStorage = newStorage;
			pHome = pStorage.Folder + "\\";  // TODO: use some System.IO.Path.Combine
		}
		#endregion

		protected string Home
		{
			get
			{
				return pHome;
			}
			set
			{
				pHome = value;
				if(pHome != null)
				{
					Load();
				}
			}
		}

		public string GetFileName(string fileName)
		{
			return pHome + fileName;
		}

		protected virtual void Load()
		{
			if(pHome != null)
			{
				Binder.BeginUpdate();
				try
				{
					foreach(AtomCollection factory in Binder)
					{
						factory.Load(pHome);
					}
				}
				finally
				{
					Binder.EndUpdate();
				}
			}
		}

		public virtual void Save()
		{
			if(pHome != null)
			{
				foreach(AtomCollection factory in Binder)
				{
					factory.Save(pHome);
				}
			}
		}

		#region Implementation of IDisposable
		private bool disposed = false;

		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		private void Dispose(bool disposing)
		{
			if(disposing && (!disposed))
			{
				Save();
			}
			DataFileName = null;
			disposed = true;
		}

		~Database()
		{
			Dispose(false);
		}
		#endregion
	}
}
