using System;
using Structure.Graph2D.IO;

namespace Structure.Graph2D.Prolog
{
	public class Writer: WriterBase
	{
		private System.IO.TextWriter _writer;
		private System.Collections.Generic.List<string> _predicates = new System.Collections.Generic.List<string>();
		
		public Writer(string fileName):
			this(new System.IO.FileStream(fileName, System.IO.FileMode.Create, System.IO.FileAccess.Write))
		{
		}
		
		public Writer(System.IO.Stream stream)
		{
			_writer = new System.IO.StreamWriter(stream, System.Text.Encoding.ASCII);
		}

		private const string QuotePrefix = "_quoted_";
		
		public static string Quote(string value)
		{
			if (String.IsNullOrEmpty(value))
			{
				return "";
			}
			else
			{
				return QuotePrefix + value;
			}
		}
		
		private static bool IsQuoted(string text, out string value)
		{
			value = text;
			if (String.IsNullOrEmpty(text))
			{
				return false;	
			}
			else if (text.StartsWith(QuotePrefix))
			{
				value = text.Substring(QuotePrefix.Length);
				return true;
			}
			else
			{
				return false;
			}
		}
		
		#region IWriter interface
		public override void Write(Edge edge)
		{
			_predicates.Add(String.Format("edge({0},{1}).", NodeId(edge.SourceId), NodeId(edge.TargetId)));
		}
		
		private string NodeId(int id)
		{
			return String.Format("n{0}", id);
		}
		
		private string TextValue(string value)
		{
			string result;
			if (String.IsNullOrEmpty(value))
			{
				return null;
			}
			else if (IsQuoted(value, out result))
			{
				return result;
			}
			else
			{
				string code = value.Replace("\n", "\\n");
				return "'" + code.Substring(0, 1).ToLower() + code.Substring(1) + "'";
			}
		}
		
		public override void Write(Node node)
		{
			_predicates.Add(String.Format("node({0}).", NodeId(node.Id)));
			string code;
			code = TextValue(node.Class);
			if (code != null)
			{
				_predicates.Add(String.Format("class({0},{1}).", NodeId(node.Id), code));
			}
			code = TextValue(node.Text);
			if (code != null)
			{
				_predicates.Add(String.Format("label({0},{1}).", NodeId(node.Id), code));
			}
		}
		
		public override void Close()
		{
			if (_writer != null)
			{
				if (_predicates != null)
				{
					// this delayed print is to make it GNU-Prolog compatible (avoid so called "discontiguous predicate").
					_predicates.Sort();
					foreach(string line in _predicates)
					{
						_writer.WriteLine(line);
					}
				}
				
				_writer.Dispose();
				_writer = null;
			}
			base.Close();
		}
		#endregion
	}
}
