using System;

namespace System.Resources
{
	public class Assembly
	{
		private static string GetResourceID(System.Reflection.Assembly container, string resourceName)
		{
			foreach(string id in container.GetManifestResourceNames())
			{
				if (id.ToUpper().EndsWith(resourceName.ToUpper()))
				{
					return id;
				}
			}
			return null;
		}

		public static System.IO.Stream GetResourceStream(string resourceName)
		{
			System.Reflection.Assembly container = System.Reflection.Assembly.GetEntryAssembly();
			string id = GetResourceID(container, resourceName);
			if (id == null)
			{
				container = System.Reflection.Assembly.GetCallingAssembly();
				id = GetResourceID(container, resourceName);
			}
			if (id == null)
			{
				// TODO: check all loaded assemblies
				return null;
			}
			return container.GetManifestResourceStream(id);
		}
		
		public static string GetResourceText(string resourceName)
		{
			System.IO.Stream stream = GetResourceStream(resourceName);
			if (stream == null)
			{
				return null;
			}
			try
			{
				System.IO.TextReader reader = System.Text.FileStream.CreateReader(stream);
				try
				{
					return reader.ReadToEnd();
				}
				finally
				{
					reader.Dispose();
				}
			}
			finally
			{
				stream.Dispose();
			}
		}
	}
}
