using System;
using System.Configuration;

namespace System.Windows.Forms
{
	public class FormLayout
	{
		public System.Drawing.Rectangle Bounds;
		public FormWindowState State;

		public void Reset()
		{
			Bounds.X = 0;
			Bounds.Y = 0;
			Bounds.Width = 0;
			Bounds.Height = 0;
			State = FormWindowState.Maximized;
		}

		public FormLayout()
		{
			Reset();
		}

		public string Hash()
		{
			Update();
			return Bounds.Left.ToString() + "," + Bounds.Top.ToString() + "," + Bounds.Width.ToString() + "," + Bounds.Height.ToString() + "," + State.ToString();
		}

		public void Update()
		{
			if (_form != null)
			{
				State = _form.WindowState;
				if (State != FormWindowState.Normal)
				{
					Bounds = _form.RestoreBounds;
				}
				else
				{
					Bounds = _form.Bounds;
				}
			}
		}

		private Form _form = null;
		public Form Form
		{
			get
			{
				return _form;
			}
			set
			{
				if (value != _form)
				{
					Update();
					_form = value;
					Update();
				}
			}
		}

		public void Attach(Form form)
		{
			if (form == null) return;

			if (!Bounds.IsEmpty)
			{
				form.WindowState = FormWindowState.Normal;
				form.StartPosition = FormStartPosition.Manual;
				form.Location = new System.Drawing.Point(Bounds.X, Bounds.Y);
				form.Bounds = Bounds;
			}
			form.WindowState = State;
			Form = form;
		}

		public void Load(ISettingsReader settings, string section)
		{
			// TODO: without Reset() default settings will be used from the attached form
			// was: Reset();
			Bounds.X = settings.ReadInt32(section + "/Left", Bounds.X);
			Bounds.Y = settings.ReadInt32(section + "/Top", Bounds.Y);
			Bounds.Width = settings.ReadInt32(section + "/Width", Bounds.Width);
			Bounds.Height = settings.ReadInt32(section + "/Height", Bounds.Height);
			State = (FormWindowState)System.Enum.Parse(typeof(FormWindowState), settings.ReadString(section + "/State", State.ToString()));
			// apply changes
			Attach(Form);
		}

		public void Save(ISettingsWriter settings, string section)
		{
			Update();
			settings.WriteInt32(section + "/Left", Bounds.X);
			settings.WriteInt32(section + "/Top", Bounds.Y);
			settings.WriteInt32(section + "/Width", Bounds.Width);
			settings.WriteInt32(section + "/Height", Bounds.Height);
			settings.WriteString(section + "/State", State.ToString());
		}
	}
}
