using System;
using System.Runtime.InteropServices;

namespace System.Windows.Forms
{
	public class ListViewHelper
	{
		[StructLayout(LayoutKind.Sequential)]
		private struct LVITEM
		{
			public uint mask;
			public int iItem;
			public int iSubItem;
			public uint state;
			public uint stateMask;
			public string pszText;
			public int cchTextMax;
			public int iImage;
			public int lParam;
			public int iIndent;
			public int iGroupId;
			public uint cColumns;
			public uint puColumns;
		}
		
		private const uint LVM_FIRST = 0x1000;
		private const uint LVM_SETITEMSTATE = (LVM_FIRST + 43);
		private const uint LVM_GETITEMSTATE = (LVM_FIRST + 44);
		private const uint LVM_GETSELECTEDCOUNT = (LVM_FIRST + 50);
		private const uint LVM_SCROLL = (LVM_FIRST + 20);
	
		private const uint LVIS_SELECTED = 0x02;
		private const uint LVIF_STATE = 0x08;
		private const uint LVIS_STATEIMAGEMASK = 0xF000;
	
		[DllImport("user32.dll", EntryPoint="SendMessage")]
		private static extern int SendItemMessage(IntPtr hWnd, uint Msg, int wParam, ref LVITEM lParam);
		
		[DllImport("user32.dll", EntryPoint="SendMessage")]
		private static extern int SendMessage(IntPtr hWnd, uint Msg, int wParam, int lParam);

		public static void SelectAll(ListView owner, bool select)
		{
			LVITEM item = new LVITEM();
			item.mask = LVIF_STATE;
			item.stateMask = LVIS_SELECTED;
			if (select)
			{
				item.state = LVIS_SELECTED;
			}
			else
			{
				item.state = 0;
			}
			SendItemMessage(owner.Handle, LVM_SETITEMSTATE, -1 /* all items */, ref item);
		}
		
		public static bool VerticalScrollBarVisible(ListView owner)
		{
			return (owner.Width - owner.ClientSize.Width) > 6 /* TODO: ? */;
		}

		[DllImport("user32.dll")]
		private static extern bool SetWindowPos(IntPtr hWnd, IntPtr hWndInsertAfter, int X, int Y, int cx, int cy, uint uFlags);
		
        private const UInt32 SWP_NOSIZE = 0x0001;
        private const UInt32 SWP_NOMOVE = 0x0002;
        private const UInt32 SWP_NOZORDER = 0x0004;
        private const UInt32 SWP_NOREDRAW = 0x0008;
        private const UInt32 SWP_NOACTIVATE = 0x0010;
        private const UInt32 SWP_FRAMECHANGED = 0x0020;  /* The frame changed: send WM_NCCALCSIZE */
        private const UInt32 SWP_SHOWWINDOW = 0x0040;
        private const UInt32 SWP_HIDEWINDOW = 0x0080;
        private const UInt32 SWP_NOCOPYBITS = 0x0100;
        private const UInt32 SWP_NOOWNERZORDER = 0x0200;  /* Don't do owner Z ordering */
        private const UInt32 SWP_NOSENDCHANGING = 0x0400;  /* Don't send WM_WINDOWPOSCHANGING */
        
        public static void ResetTopItem(ListView owner)
		{
			// TODO: this does nothing
			// SendMessage(owner.Handle, LVM_SCROLL, 0, owner.Height);
			/* TODO: following doesn't work either
			Control oldParent = owner.Parent;
			DockStyle oldDock = owner.Dock;
			//owner.Parent = null;
			owner.Dock = DockStyle.None;
			owner.Height = 30;
			owner.Dock = oldDock;
			owner.Parent = oldParent;
			*/
			/* TODO: following doesn't work either + looses selected items
			bool oldScrollable = owner.Scrollable;
			owner.Scrollable = false;
			owner.Scrollable = true;
			int w = owner.Width;
			int h = owner.Height;
			DockStyle oldDock = owner.Dock;
			owner.Dock = DockStyle.None;
			SetWindowPos(owner.Handle, IntPtr.Zero, 0, 0, w, h - 1, SWP_NOMOVE | SWP_NOZORDER);
			owner.Scrollable = oldScrollable;
			owner.Dock = oldDock
			SetWindowPos(owner.Handle, IntPtr.Zero, 0, 0, w, h, SWP_NOMOVE | SWP_NOZORDER);
			*/
		}
	}
}
